@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001-8 John Elliott <jce@seasip.demon.co.uk>           *
+ *    Copyright (C) 2001-2020 John Elliott <seasip.webmaster@gmail.com>    *
  *                                                                         *
  *    Modifications to add dsk_dirty()                                     *
  *    (c) 2005 Philip Kendall <pak21-spectrum@srcf.ucam.org>               *
@@ -58,7 +58,7 @@
 extern "C" {
 #endif
 
-#define LIBDSK_VERSION "1.3.3"
+#define LIBDSK_VERSION "1.5.16"
 
 /************************* TYPES ********************************/
 
@@ -100,6 +100,8 @@ typedef const char *  dsk_cchar_t;	/* Const char * */
 #define DSK_ERR_TIMEOUT  (-29)  /* Communications timed out */
 #define DSK_ERR_UNKRPC   (-30)  /* RPC server does not recognise function */
 #define DSK_ERR_BADMEDIA (-31)	/* Unsuitable media for drive */
+#define DSK_ERR_CORRUPT  (-32)	/* Disk image file is corrupt */
+#define DSK_ERR_NULLOPT  (-33)  /* Option is valid, but has no value */
 #define DSK_ERR_UNKNOWN  (-99)	/* Unknown error */
 
 /* Is this error a transient error, that may be cleared by a retry? */
@@ -112,8 +114,11 @@ typedef enum
 	SIDES_ALT, 	/* Track n is cylinder (n/heads) head (n%heads) */
 	SIDES_OUTBACK, 	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
 			             (head 1) 39,38,37,...,2,1,0 */
-	SIDES_OUTOUT	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
+	SIDES_OUTOUT,	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
 			             (head 1) 0,1,2,3,...37,38,39 */
+	SIDES_EXTSURFACE/* As SIDES_ALT, but sectors on head 1 identify
+			 * as head 0, with numbers in sequence 
+			 * eg: Head 0 has sectors 1-9, head 1 has 10-18 */
 } dsk_sides_t;
 
 typedef enum
@@ -149,9 +154,42 @@ typedef enum
 	FMT_MBEE400,	/* 10 sectors, 80 tracks, 1 side */
 	FMT_MGT800,     /* 10 sectors, 80 tracks, 2 sides out and out */
 	FMT_TRDOS640,	/* 16 sectors,  256 bytes/sector, 2 sides */
+	FMT_AMPRO200,	/* 10 sectors, 512 bytes/sector, 1 side */
+	FMT_AMPRO400D,	/* 10 sectors, 512 bytes/sector, 2 sides */
+	FMT_AMPRO400S,	/* 5 sectors, 1024 bytes/sector, 1 side */
+	FMT_AMPRO800,	/* 5 sectors, 1024 bytes/sector, 2 sides */
+	FMT_1200K,	/* 15 sectors, 2 sides  */
+	FMT_MAC400,	/* Apple GCR 400k */
+	FMT_MAC800,	/* Apple GCR 800k */
 	FMT_UNKNOWN = -1
 } dsk_format_t;
 
+typedef enum
+{
+	/* Low byte of dg_fm: Recording mode */
+	RECMODE_MASK       = 0x00FF,
+	RECMODE_MFM        = 0x0000,	
+	RECMODE_FM         = 0x0001,
+/* Annoyingly, LDBS recording modes match LibDsk ones except for MFM, 
+ * which is 2 in LDBS rather than 0. To avoid any possible conflict, 
+ * don't use 2 for any future recording mode in this structure; use
+ * 3+ for MFM variants */
+
+/* Recording modes 0x10-0x2F are GCR, defined as the GCR format byte 
+ * (byte 0x51 of an Apple Disk Copy file) masked with 0x1F, plus 0x10.
+ * These fall into the range RECMODE_GCR_FIRST - RECMODE_GCR_LAST */
+	RECMODE_GCR_FIRST  = 0x0010,
+	RECMODE_GCR_MAC    = 0x0012,      /* Macintosh 400k / 800k GCR */
+	RECMODE_GCR_PRODOS = 0x0014,      /* Apple IIgs Prodos 800k GCR */
+	RECMODE_GCR_LISA   = 0x0022,      /* Apple Lisa 400k GCR */
+	RECMODE_GCR_LAST   = 0x002F,
+
+	/* High byte of dg_fm: Other data recording flags */
+	RECMODE_FLAGMASK   = 0xFF00,
+	RECMODE_COMPLEMENT = 0x0100	
+	
+} dsk_recmode_t;
+
 /* DSK_GEOMETRY holds information used to convert physical to/from logical
  * sectors and to access the disc */
 typedef struct
@@ -165,8 +203,8 @@ typedef struct
 	dsk_rate_t	dg_datarate;	/* Data rate */
 	dsk_gap_t	dg_rwgap;	/* Read/write gap length */
 	dsk_gap_t	dg_fmtgap;	/* Format gap length */
-	int		dg_fm;		/* FM mode? The only thing I know that uses FM mode
-					 * is the BBC Micro DFS format. */
+	int		dg_fm;		/* Really a dsk_recmode_t, kept as
+					 * int for backward compatibility  */
 	int		dg_nomulti;	/* Disable multitrack? */
 	int		dg_noskip;	/* Set to 0 to skip deleted data */
 } DSK_GEOMETRY;
@@ -216,9 +254,12 @@ LDPUBLIC32 char * LDPUBLIC16 dsk_strerror(/* in */ dsk_err_t err);
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_stdformat(DSK_GEOMETRY *self, dsk_format_t formatid,
 			dsk_cchar_t *name, dsk_cchar_t *desc);
 
-/* Convert sector size to a physical sector shift as used by the controller.
- * To go the other way, size = 128 << psh  */
+/* Convert sector size to a physical sector shift as used by the controller. */
 LDPUBLIC32 unsigned char LDPUBLIC16 dsk_get_psh(size_t sector_size);
+/* Convert physical sector shift back to sector size.
+ * 0-8: Returns 128 << psh.
+ *   9 and higher: Returns 32768. */
+LDPUBLIC32 size_t LDPUBLIC16 dsk_expand_psh(unsigned char psh);
 
 /* Register callbacks for LibDsk functions to display information on the
  * screen. */
@@ -379,6 +420,14 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_pcwgeom(DSK_GEOMETRY *self, const unsigned c
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_cpm86geom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_aprigeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_opusgeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_dfsgeom(DSK_GEOMETRY *self, const unsigned char *sector0, const unsigned char *sector1);
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_hfsgeom(DSK_GEOMETRY *self, const unsigned char *superblock);
+
+/* Tries all the above, in approximate order of detectability. This is 
+ * intended to assist in parsing flat file disc images that contain no
+ * metadata, so the first 512 bytes are all you have to determine the
+ * format. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_bootsecgeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 
 /* Read a random sector header from current track */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_psecid(DSK_PDRIVER self, const DSK_GEOMETRY *geom,
@@ -414,6 +463,10 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_pseek(DSK_PDRIVER self, const DSK_GEOMETRY
  * Else sets (*drvname) to null. */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_type_enum(int idx, char **drvname);
 
+/* If "index" is in range, returns the human-readable description of the 
+ * n'th driver name in (*drvdesc).  Else sets (*drvdesc) to null. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_typedesc_enum(int idx, char **drvdesc);
+
 /* If "index" is in range, returns the n'th compressor name in (*compname).
  * Else sets (*drvname) to null. */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_comp_enum(int idx, char **compname);
@@ -467,18 +520,31 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_map_itod(unsigned int n, DSK_PDRIVER *ptr);
  *  * all the memory used by the mapping */
 LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_map_delete(unsigned int index);
 
-
+/* Copy one entire disk image to another. Currently this can only be done if 
+ * both of them are based internally on LDBS; DSK_ERR_NOTIMPL will be returned 
+ * otherwise. 
+ * 
+ * geom can be null. If it is not null, it will be used when converting 
+ * from file formats that don't specify their own geometry (such as raw)
+ */
+LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_copy(DSK_GEOMETRY *geom, 
+				DSK_PDRIVER source, DSK_PDRIVER dest);
 
 /* Define this to print on the console a trace of all mallocs */
 #undef TRACE_MALLOCS 
 #ifdef TRACE_MALLOCS
 void *dsk_malloc(size_t size);
+void *dsk_remalloc(void *ptr, size_t size);
 void  dsk_free(void *ptr);
 #else
 #define dsk_malloc malloc
+#define dsk_realloc realloc
 #define dsk_free free
 #endif
 
+/* Helper method: Copy string 's' to a dsk_malloced buffer */
+LDPUBLIC32 char * LDPUBLIC16 dsk_malloc_string(const char *s);
+
 #ifdef __cplusplus
 }
 #endif
