@@ -1115,6 +1115,6 @@ void SdSdl::init_wk(const GWSParams *wsp, GWSResult *wsr)
 		wsr->points[i].x = m_globals->m_SIZ_TAB[i].x;
 		wsr->points[i].y = m_globals->m_SIZ_TAB[i].y;
 	}
-	wsr->data[0] = m_globals->m_surface->w - 1;	
-	wsr->data[1] = m_globals->m_surface->h - 1;	
+	wsr->data[0] = m_globals->m_requestW - 1;	
+	wsr->data[1] = m_globals->m_requestH - 1;	
 }
