@@ -167,7 +167,7 @@ Uint32 SdSdl::text_move(Uint32 form_w, Uint32 char_w, Uint32 src_x)
  * Returns:	none						*
  ****************************************************************/
 
-Uint32 SdSdl::text_italic(Uint32 char_w, Uint32 rows, Uint32 buf_w)
+void SdSdl::text_italic(Uint32 char_w, Uint32 rows, Uint32 buf_w)
 {
 	Uint8 *src, *dest;
 	Uint8 leftx, rightx;
