@@ -2,7 +2,7 @@
 
     SdlTerm v1.01 - Terminal emulation in an SDL framebuffer
 
-    Copyright (C) 1996, 2001, 2006  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001, 2006  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -31,16 +31,16 @@ static Uint8 fontDef[] =
 
 SdlTerm::SdlTerm(SDL_Surface *s, SDL_Rect *bounds) : SdlContext(s, bounds)
 {
-	int n, m;
 	SXLOG("SdlTerm::SdlTerm()");
-	for (n = 0; n < SDLK_LAST; n++)
+	for (int n = 0; n < SDLK_LAST; n++)
 	{
-		for (m = 0; m < 8; m++)
+		for (int m = 0; m < 8; m++)
 		{
 		        m_keyMap[m][n] = new std::string;
 		}	
 	}
 	m_font    = NULL;
+	m_curVisible = false;
 	reset();
 	resetInput();
 	SXLOG("SdlTerm::SdlTerm() done");
@@ -49,18 +49,16 @@ SdlTerm::SdlTerm(SDL_Surface *s, SDL_Rect *bounds) : SdlContext(s, bounds)
 
 SdlTerm::~SdlTerm()
 {
-	int n, m;
-
 	SXLOG("SdlTerm::~SdlTerm()");
 	
 	if (m_font) 
 	{
-		delete m_font;
+		delete [] m_font;
 		m_font = NULL;
 	}
-	for (n = 0; n < SDLK_LAST; n++)
+	for (int n = 0; n < SDLK_LAST; n++)
 	{
-		for (m = 0; m < 8; m++)
+		for (int m = 0; m < 8; m++)
 		{
 		        delete m_keyMap[m][n];
 		}	
@@ -144,7 +142,7 @@ void SdlTerm::setFont(Uint8 *psf)
 	if (!buf) return;
 
 	memcpy(buf, psf, 4 + 256 * psf[3]);
-	if (m_font) delete m_font;
+	if (m_font) delete [] m_font;
 	m_font = buf;
 
 	m_charmax = 256;
@@ -597,6 +595,9 @@ void SdlTerm::bel(void)
 
 void    SdlTerm::moveCursor(int x, int y)
 {
+	if (x < 0) x = 0;
+	if (y < 0) y = 0;
+
 	hideCursor();
 	m_cx = x;
 	m_cy = y;
@@ -820,10 +821,10 @@ void SdlTerm::scrollDown(void)
 {
         SDL_Rect sr, dr;
 
-        sr.x += m_bounds.x;
-        sr.y += m_bounds.y;
 
         sr = m_windowP;
+        sr.x += m_bounds.x;
+        sr.y += m_bounds.y;
         sr.h -= m_charh;
         dr = sr;
         dr.y += m_charh;
@@ -876,7 +877,7 @@ void SdlTerm::glyph(unsigned char c)
 	if (m_reverse) for (n = 0; n < m_charh; n++) intbm[n] = ~intbm[n];
 
 	rawGlyph(m_windowP.x + m_cx * 8, m_windowP.y + m_cy * m_charh, intbm, 0);
-	delete intbm;
+	delete [] intbm;
 }
 
 
