@@ -2,7 +2,7 @@
 
     JOYCE v2.1.3 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001-3  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-3  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -1808,10 +1808,10 @@ void AnneAccel::patch(byte *rosanne)
 
 // See if the current ROM checksum is correct. 
 			sum = ~checksum(rosanne, &chksum);
-			if ((chksum[0] == (sum      ) & 0xFF) &&
-			    (chksum[1] == (sum >>  8) & 0xFF) &&
-			    (chksum[2] == (sum >> 16) & 0xFF) &&
-			    (chksum[3] == (sum >> 24) & 0xFF)) fixsum = 1; 
+			if ((chksum[0] == ((sum      ) & 0xFF)) &&
+			    (chksum[1] == ((sum >>  8) & 0xFF)) &&
+			    (chksum[2] == ((sum >> 16) & 0xFF)) &&
+			    (chksum[3] == ((sum >> 24) & 0xFF))) fixsum = 1;
 // Poke in the enable / disable sequence.	
 			if (isEnabled())
 			{
