@@ -2,7 +2,7 @@
 
     JOYCE v2.1.0 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001-2  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -620,7 +620,7 @@ void AnneTerminal::border(byte forced)
 	Uint8 g;
 	int border_idx = m_indices[m_colourReg & 0x1F];
 
-        if (!m_visible || !m_sysVideo || !(m_scrEnabled & 1)) return;
+        if (!m_sysVideo) return;
 
         if (m_colourReg & 0x80)  /* Colours inverted */
         {
@@ -632,6 +632,7 @@ void AnneTerminal::border(byte forced)
                 gl_pcwink[0] = m_pcwInk[0] = m_pcwDInk[0];
                 gl_pcwink[1] = m_pcwInk[1] = m_pcwDInk[1];
 	}
+        if (!m_visible || !(m_scrEnabled & 1)) return;
         if (!forced && !m_borderInvalid) return;
         m_borderInvalid = 0;
 
