@@ -2,7 +2,7 @@
  
     JOYCE v1.90 - Amstrad PCW emulator
  
-    Copyright (C) 1996, 2000, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2000, 2001  John Elliott <seasip.webmaster@gmail.com>
  
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
