@@ -2,7 +2,7 @@
 
     JOYCE v1.90 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -150,6 +150,12 @@ bool JoyceAsic::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *cur)
         parseInteger(doc, ns, cur, "cpuspeed", &r);
         if (r) m_sys->m_cpu->setSpeed(r);
 
+       parseInteger(doc, ns, cur, "jdos", &r);
+	if (r == JM_PROMPT || r == JM_ALWAYS || r == JM_NEVER)
+	{
+		m_sys->setJDOSMode((JDOSMode)r);
+	}
+
 	xmlChar *s = xmlGetProp(cur, (xmlChar *)"model");
 	if (s)
 	{
@@ -178,6 +184,9 @@ bool JoyceAsic::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *cur)
 		case PCW9000P: xmlSetProp(cur, (xmlChar *)"model", (xmlChar *)"9512+"); break;
 		case PCW10:    xmlSetProp(cur, (xmlChar *)"model", (xmlChar *)"9256"); break;
 	}
+	r = m_sys->getJDOSMode();
+        storeInteger(doc, ns, cur, "jdos", r);
+
 	return true;
 }
 
@@ -218,6 +227,9 @@ UiEvent JoyceAsic::settings(UiDrawer *d)
                 uim.add(new UiTextInput(SDLK_j, "  JOYCE data directory: ________________________________  ",d));
                 uim.add(new UiSeparator(d));
 #endif
+		uim.add(new UiCommand(SDLK_f, "  File access...", d));
+                uim.add(new UiSeparator(d));
+
                 uim.add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
                 d->centreContainer(uim);
                 uim.setSelected(sel);
@@ -243,6 +255,10 @@ UiEvent JoyceAsic::settings(UiDrawer *d)
 			case SDLK_j:
 				setHomeDir(((UiTextInput &)uim[9]).getText());
 				break;
+			case SDLK_f:
+				uie = setFileAccess(d);
+				if (uie == UIE_QUIT) return uie;
+				break;
 			case SDLK_ESCAPE:
 				m_sys->setModel(m);
         			if (r) m_sys->m_cpu->setSpeed(r);
