@@ -2,7 +2,7 @@
 
     JOYCE v2.1.0 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -61,7 +61,7 @@
 // This is for the default "Prestige Pica 10" wheel
 // It can be configured by editing joycehw.xml (no UI is provided, as
 // it is very unlikely that one will be needed).
-static unsigned char gl_wheel[128] = 
+static char gl_wheel[128] = 
 {
 	'#', '.', '<', '[', '=', '9', '8', '7',	// 00-07
 	'6', '5', '0', '4', '3', '2', '1', ',',	// 08-0F
@@ -348,12 +348,11 @@ string JoyceDaisy::getTitle(void)
 bool JoyceDaisy::storePrintWheel(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
         bool found = false;	
-	int n;
         xmlNodePtr output, newOutput, pin;
 	char buf[30];
 
         newOutput    = xmlNewNode(ns, (xmlChar *)"printwheel");
- 	for (n = 0; n < sizeof(m_wheel); n++) 
+ 	for (unsigned n = 0; n < sizeof(m_wheel); n++) 
 	{
 		if (m_wheel[n]) 
 		{
@@ -405,7 +404,8 @@ bool JoyceDaisy::parsePrintWheel(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 			{ 
 				idx = atoi(i); 
 				ch  = atoi(c);
-				if (idx >= 0 && idx < sizeof(m_wheel)) m_wheel[idx] = ch;
+				if (idx >= 0 && idx < (int)sizeof(m_wheel)) 
+					m_wheel[idx] = ch;
 			}
 			if (i) xmlFree(i);
 			if (c) xmlFree(c);
@@ -471,6 +471,8 @@ bool JoyceDaisy::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
         xmlSetProp(cur, (xmlChar *)"fontsize", (xmlChar *)buf);
         switch(m_format)
         {
+		/* Should never happen */
+                case PNG:  xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"PNG");
                 case PS:   xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"PS");
 #ifdef HAVE_WINDOWS_H
                 case GDI:  xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"GDI");
@@ -629,6 +631,8 @@ bool JoyceDaisy::startPage(void)
 {
         switch(m_format)
         {
+		case PNG: // Should never happen
+			return false;
                 case PS:  return startPagePS();
 #ifdef HAVE_WINDOWS_H
                 case GDI:
@@ -734,6 +738,7 @@ bool JoyceDaisy::endPage(void)
 {
         switch(m_format)
         {
+		case PNG: break;
                 case PS:  return endPagePS();
 #ifdef HAVE_WINDOWS_H
                 case GDI: m_drawing_page = false;
@@ -748,10 +753,11 @@ void JoyceDaisy::putChar(char c, float xf, float yf)
 {
         switch(m_format)
         {
-                case PS:  charPS(c, xf, yf);
+		case PNG: break;
+                case PS:  charPS(c, xf, yf); break;
 #ifdef HAVE_WINDOWS_H
 // [JOYCE v2.1.1: Convert from 120/192dpi to 360dpi]
-                case GDI: m_gdiPrint.putChar(xf * 3.0, yf * 1.875, c);
+                case GDI: m_gdiPrint.putChar(xf * 3.0, yf * 1.875, c); break;
 #endif
         }
 }
