@@ -1,8 +1,8 @@
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.2.5a - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001, 2012  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -32,11 +32,12 @@
 #include "PcwFileList.hxx"
 //#include "JoyceDirChooser.hxx"
 
-/* Define png_jmpbuf() in case we are using a pre-1.0.6 version of libpng */
+/* Define png_jmpbuf() in case we are using an early version of libpng */
 #ifndef png_jmpbuf
 #  define png_jmpbuf(png_ptr) png_ptr->jmpbuf
 #endif
 
+
 /* Adjustments to matrix printer. The K_ADJUST is how many "ticks" the 
  * print head needs to get moving, and the L_ADJUST is how many it takes
  * to stop. */
@@ -332,7 +333,7 @@ void JoyceMatrix::dots(void)
 
 static void matrix_png_error(png_structp png_ptr, png_const_charp error_msg)
 {
-	JoyceMatrix *m = (JoyceMatrix *)(png_ptr->error_ptr);	
+	JoyceMatrix *m = (JoyceMatrix *)png_get_error_ptr(png_ptr);
 	UiDrawer *d = gl_sys->m_termMenu;
         UiMenu uim(d);
 	string sl1,sl2;
@@ -346,13 +347,13 @@ static void matrix_png_error(png_structp png_ptr, png_const_charp error_msg)
         d->centreContainer(uim);
 	uim.eventLoop();
 
-	longjmp(png_ptr->jmpbuf, 1);
+	longjmp(png_jmpbuf(png_ptr), 1);
 }
 
 
 static void matrix_png_warn(png_structp png_ptr, png_const_charp warning_msg)
 {
-	JoyceMatrix *m = (JoyceMatrix *)(png_ptr->error_ptr);	
+	JoyceMatrix *m = (JoyceMatrix *)png_get_error_ptr(png_ptr);
 	joyce_dprintf("PNG warning: %s\n", warning_msg);
 }
 
