@@ -1,8 +1,8 @@
 /************************************************************************
 
-    JOYCE v2.1.11 - Amstrad PCW emulator
+    JOYCE v2.2.10 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001-9  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-9, 2014  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -43,13 +43,13 @@ JoyceMenuTermPcw::~JoyceMenuTermPcw()
 
 void JoyceMenuTermPcw::menuDraw(void)
 {
-	char menustr[101];
+	char menustr[201];
 	double speed = m_sys->m_cpu->getSpeed(false);
 	SDL_Rect &rc = getSysVideo()->getBounds();
 
 	m_term->fillRect(0, 0, rc.w, UI_CHAR_H,
 			 m_term->getForeground());
-        sprintf(menustr,"JOYCE v%d.%d.%d  Copyright 1996,2001-10, John Elliott  "
+        sprintf(menustr,"JOYCE v%d.%d.%d  Copyright 1996,2001-17, John Elliott  "
                         "Compiled with SDL version %d.%d.%d  Speed=%.2f%%",
                         BCDVERS >> 8,
                         (BCDVERS >> 4) & 0x0F,
@@ -58,6 +58,7 @@ void JoyceMenuTermPcw::menuDraw(void)
                         SDL_MINOR_VERSION,
                         SDL_PATCHLEVEL,
 			speed );
+	menustr[100] = 0;
 	m_term->setReverse();
 	m_term->home();
 	(*m_term) << menustr;
