@@ -2,7 +2,7 @@
 
     JOYCE v1.90 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -61,7 +61,15 @@ static Uint16 gl_defaultMap[] = {
 	SDLK_7,         5, 2,   5, 2,
 	SDLK_8,         5, 1,   5, 1,
 	SDLK_9,         4, 2,   4, 2,
+/* [JOYCE 2.2.12] Under Linux, SDLK_BACKSLASH is the key between Shift and Z.
+ *                Under Windows, it's the key left of Carriage Return.
+ */
+#ifdef HAVE_WINDOWS_H
+	SDLK_BACKSLASH, 2,  8,  2,  8,
+#else
 	SDLK_BACKSLASH, 2, 64,  2, 64,
+#endif
+	SDLK_LESS,	2, 64,  2, 64,
 	SDLK_SEMICOLON, 3, 32,  3, 32,
 	SDLK_EQUALS,    3, 1,   3, 1,
 	SDLK_KP0,       0, 2,   0, 2,
@@ -371,7 +379,7 @@ int JoycePcwKeyboard::handleEvent(SDL_Event &e)
 		b = (e.type == SDL_KEYDOWN);
 		keysym = e.key.keysym.sym; 
 
-///		printf("Key event: b=%d keysym=%d\n", b, keysym);
+//		printf("Key event: b=%d keysym=%d\n", b, keysym);
 	}
 
 	if (keysym == SDLK_F2 || keysym == SDLK_F4 || 
