@@ -1,8 +1,8 @@
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.2.8 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2, 2016  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,12 +14,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
+
 #include "Joyce.hxx"
 #include "UiContainer.hxx"
 #include "UiMenu.hxx"
@@ -28,6 +29,9 @@
 #include "UiSetting.hxx"
 #include "UiCommand.hxx"
 
+#define MAXLINE 288
+#define MAXSTATE ((MAXLINE * 8) + 8)
+
 JoycePcwMouse::JoycePcwMouse(JoyceSystem *s) :
         PcwDevice("mouse", "pcw"), PcwInput(s)
 {
@@ -60,13 +64,18 @@ void JoycePcwMouse::reset(void)
 	m_ymin = 44;
 	m_ymax = 556;
 // This scaling works well on normal JOYCE with Stop Press and MD3
-	m_xscale = 2;
+	if (m_mouseType == MT_ESPEN) m_xscale = 1;
+	else			     m_xscale = 2;
 	m_yscale = 2;
 	getMousePos(&m_ox, &m_oy);
 // Disable patches
 
 	m_mpmx = m_spmx = m_spmy = m_md3mx = m_md3my = m_md3mf1 = m_md3mf2 = 0;
 
+// Reset lightpen counter. Start it off in the middle of a frame.
+	m_lpc = MAXSTATE / 2;
+	m_lcount_last = 0;
+
 	updateSettings();
 }
 
@@ -155,7 +164,7 @@ int JoycePcwMouse::handleEvent(SDL_Event &e)
 	return 1;
 	
 }
-
+extern int tr;
 
 byte JoycePcwMouse::in(word port)
 {
@@ -166,6 +175,9 @@ byte JoycePcwMouse::in(word port)
 
 	if (m_autoPatch && (m_patchApplied < 2)) checkPatch();
 
+//	printf("JoycePcwMouse::in(%x) mouseType=%d\n", port, m_mouseType);
+//	tr = 2;
+
 	switch(m_mouseType)
 	{
 /* According to the PCW Plus Tips Collection, AMX mouse interface is:
@@ -233,6 +245,62 @@ byte JoycePcwMouse::in(word port)
 				   if (y & 4) x &= 0xFE;
 				   return x;
 		}
+		break;
+
+		case MT_ESPEN:	
+			if (port == 0xA6 || port == 0xA7)
+			{
+			        int col, row;
+				unsigned short count = 0;
+
+        			getMousePos(&col, &row);
+
+/* count0 gets updated when the lightpen is triggered. Otherwise it retains
+ * its last value, m_lcount_last */
+
+				count = m_lcount_last & 0xFFF;
+
+/* m_lpc says what state we're in. 
+ * 0-7: Start of frame. Return VSYNC set for the first 4 reads, then clear 
+ *     for the next 4 */
+				if (m_lpc < 4)
+				{
+					count |= 0x4000;
+				}
+				else if (m_lpc < 8)
+				{
+					/* do nothing */
+				}
+/* 8 - (MAXLINE * 8)+7: On a scanline. Return HSYNC set on the second read */
+				else if (m_lpc < (MAXLINE * 8 + 8))
+				{
+					int y = ((m_lpc / 8) - 37);
+					/* In horizontal retrace? */
+					if ((m_lpc % 8) == 1)
+					{
+						count |= 0x2000;
+/* If lightpen triggered on the current row, encode column */
+						if (y == row)
+						{
+							count += (1568 - 2*col);
+							count &= 0xFFF;
+						}
+					}
+				}
+/* (MAXLINE *8) + 8 - MAXSTATE: Back into VSYNC */
+				else 
+				{
+					count |= 0x4000;
+				}
+				m_lcount_last = count & 0xFFF;
+				++m_lpc;				
+				if (m_lpc >= MAXSTATE) m_lpc = 0;
+
+				if (port == 0xA6) 
+					return (count & 0xff);
+				if (port == 0xA7) 
+					return (count >> 8);
+			}
 
 		default: break;
 	}
@@ -240,6 +308,8 @@ byte JoycePcwMouse::in(word port)
 }
 
 
+
+
 //
 // Settings functions.
 // See if this device has user-settable options. If it does, populates
@@ -248,7 +318,7 @@ byte JoycePcwMouse::in(word port)
 bool JoycePcwMouse::hasSettings(SDLKey *key, string &caption)
 {
 	*key     = SDLK_m;
-	caption = "  Mouse"; 
+	caption = "  Mouse / lightpen  "; 
 	return true;
 }
 //
@@ -265,11 +335,12 @@ UiEvent JoycePcwMouse::settings(UiDrawer *d)
 	{
 		UiMenu uim(d);
 
-		uim.add(new UiLabel("    Mouse settings    ", d))
+		uim.add(new UiLabel(" Pointing device settings    ", d))
 		   .add(new UiSeparator(d))
                    .add(new UiSetting(SDLK_n, true, "  No mouse connected   ",d))
 	           .add(new UiSetting(SDLK_a, false,"  AMX-type mouse       ",d))
-                   .add(new UiSetting(SDLK_k, false,"  Kempston mouse       ",d));
+                   .add(new UiSetting(SDLK_k, false,"  Kempston mouse       ",d))
+                   .add(new UiSetting(SDLK_e, false,"  Electric Studio lightpen  ",d));
 
 		if (isEnabled())
 		{
@@ -285,6 +356,10 @@ UiEvent JoycePcwMouse::settings(UiDrawer *d)
 			{
 				((UiSetting &)uim[4]).setChecked(true);
 			}
+			if (m_mouseType == MT_ESPEN) 
+			{
+				((UiSetting &)uim[5]).setChecked(true);
+			}
 		}
 		uim.add(new UiSeparator(d))
                    .add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
@@ -319,6 +394,10 @@ UiEvent JoycePcwMouse::settings(UiDrawer *d)
 				       m_mouseType = MT_KEMPSTON; 
 				       m_autoPatch = false;
 				       break;
+			case SDLK_e:   enable(true); 
+				       m_mouseType = MT_ESPEN;
+				       m_autoPatch = false;
+				       break;
 			case SDLK_p:   m_autoPatch = true;  break;		
 			case SDLK_o:   m_autoPatch = false; break;		
 			default: break;
@@ -386,6 +465,7 @@ bool JoycePcwMouse::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
         {
                 if      (!strcmp(s, "amx"))      m_mouseType = MT_AMX;
                 else if (!strcmp(s, "kempston")) m_mouseType = MT_KEMPSTON;
+                else if (!strcmp(s, "es_pen"))   m_mouseType = MT_ESPEN;
                 else joyce_dprintf("Mouse type '%s' is unknown - ignored\n",s);
 		xmlFree(s);
         }
@@ -408,6 +488,7 @@ bool JoycePcwMouse::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
         {
                 case  MT_AMX:      xmlSetProp(cur, (xmlChar *)"type", (xmlChar *)"amx");      break;
                 case  MT_KEMPSTON: xmlSetProp(cur, (xmlChar *)"type", (xmlChar *)"kempston"); break;
+                case  MT_ESPEN:    xmlSetProp(cur, (xmlChar *)"type", (xmlChar *)"es_pen"); break;
     		default:	   break;
 	}
 	return storeEnabled(doc, ns, cur);
