@@ -3,7 +3,7 @@
 
     JOYCE v2.1.0 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001-2  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -322,6 +322,7 @@ bool JoyceSystem::deinitVideo(void)
 	// [v1.9.3] Shut down sound before releasing SDL
         ((JoycePcwTerm *)m_defaultTerminal)->deinitSound();
 	PcwSystem::deinitVideo();
+	return true;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
