@@ -2,7 +2,7 @@
 
     JOYCE v1.90 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -35,10 +35,96 @@ void LoadBoot(void);
 
 extern int daisy_int;
 
-/* Variables controlling JOYCE trace code */
+/* Hacked-together implementation of the pcwlink interface, just as a proof
+ * of concept
+
+static signed char pcwlink_channel = -1;
+
+static void pcwlink_log(const char *s, ...)
+{
+	char buf[200];
+	va_list ap;
+	FILE *fp;
+
+	va_start(ap, s);
+	vsprintf(buf, s, ap);
+	fp = fopen("pcwlink.log", "a");
+	fprintf(fp, "%s\n", buf);
+	fclose(fp);
+	va_end(ap);
+
+}
+
+static void open_pcwlink()
+{
+	FILE *fp = fopen("pcwlink.ch", "r+b");
+	if (!fp)
+	{
+		pcwlink_channel = 0;
+		fp = fopen("pcwlink.ch", "wb");
+		fwrite(&pcwlink_channel, 1, 1, fp);
+		fclose(fp);
+	}
+	else
+	{
+		fread(&pcwlink_channel, 1, 1, fp);
+		pcwlink_channel ^= 1;	
+		fseek(fp, 0, SEEK_SET);
+		fwrite(&pcwlink_channel, 1, 1, fp);
+		fclose(fp);
+	}
+	pcwlink_log("[%d] PCWLink channel: %d", getpid(), pcwlink_channel);
+}
+
+static void write_pcwlink(Z80 *r, byte v)
+{
+	FILE *fp;
+	if (pcwlink_channel == -1) open_pcwlink();
+
+	if (pcwlink_channel == 0) fp = fopen("pcwlink.0", "wb");
+	else			  fp = fopen("pcwlink.1", "wb");
+	fwrite(&v, 1, 1, fp);
+	fclose(fp);
+	if (r->pc == 0x4167) 
+		pcwlink_log("[%d] PCWLink write @ %04x %04x:   %d", getpid(), r->pc, r->iy, v & 1);
+	else if (r->pc == 0x4151) 
+		pcwlink_log("[%d] PCWLink write @ %04x %04x: %d  ", getpid(), r->pc, r->iy, v & 2);
+	else	pcwlink_log("[%d] PCWLink write @ %04x %04x: %d %d", getpid(), r->pc, r->iy, v & 2, v & 1);
+}
+
+
+static byte read_pcwlink(Z80 *r)
+{
+	byte v;
+	FILE *fp;
+
+	if (pcwlink_channel == -1) open_pcwlink();
+
+	if (pcwlink_channel == 0) fp = fopen("pcwlink.1", "rb");
+	else			  fp = fopen("pcwlink.0", "rb");
+
+	if (fp == NULL) v = 0xFF;
+	else
+	{
+		fread(&v, 1, 1, fp);
+		fclose(fp);
+	}
+	if (r->pc == 0x4170) 
+		pcwlink_log("[%d] PCWLink read  @ %04x %04x: %d  ", getpid(), r->pc, r->iy, (~v) & 2);
+	else if (r->pc == 0x4185) 
+		pcwlink_log("[%d] PCWLink read  @ %04x %04x:   %d", getpid(), r->pc, r->iy, (~v) & 1);
+	else
+		pcwlink_log("[%d] PCWLink read  @ %04x %04x: %d %d", getpid(), r->pc, r->iy, (~v) & 2, (~v) & 1);
+	
+//	return ~(((v << 1) & 0xFE) | ((v >> 1) & 1));
+	return ~v;
+}
+*/
+
+// Variables controlling JOYCE trace code 
 int tr = 0;
 static int id = 0;
-
+/*
 static void daisyIn(byte hi, byte lo, word pc, byte a, byte l, word sp)
 {
 	int n;
@@ -54,7 +140,7 @@ static void daisyOut(byte hi, byte lo, byte v, word pc)
 {
 	fprintf(stderr, "DAISY: [%04x] OUT %02x%02x, %02x\n", pc, hi, lo, v);
 }
-
+*/
 
 JoyceZ80::JoyceZ80(JoyceSystem *s)
 {
@@ -85,12 +171,19 @@ void JoyceZ80::main_every(void)
 		}
 	
 */
+
 /*
+	if (pc >= 0x3e88 && pc <= 0x4130 && pcwlink_channel == 0)
+		pcwlink_log("[%d] PC=%04x %02x AF=%02x:%02x BC=%04x DE=%04x HL=%04x IX=%04x IY=%04x\n",
+			getpid(), pc, fetch(pc), a,f, getBC(), getDE(), getHL(), 
+			ix, iy);
+*/
+
  // Uncomment & modify this code for excruciating traces of what the Z80 is
  // doing.
 	{
 
-//		if (pc == 0xB66)	// Expander
+//		if (pc == 0x11c9)	// Expander
 //		{
 //			if (m_sys->getDebug()) tr = 1;
 //		}
@@ -98,7 +191,12 @@ void JoyceZ80::main_every(void)
 //		if (id == 500 && tr == 2) tr = 0;
 //
 //		if (tr == 1 && pc == 0x173) { printf("READ BYTE "); tr = 2; }
-*/		if (pc == 0xD461) tr = 2;
+//		if (pc == 0x11c9) tr = 2;
+
+//		if (pc == 0xbcee && c == 2) tr = 1;
+//		if (pc == 0xbcf1 && tr == 1) tr = 2;
+
+/*
 		if (tr == 2)
 		{
 			++id;
@@ -107,12 +205,13 @@ void JoyceZ80::main_every(void)
 			ix, iy);
 			fflush(stdout);
 		}
-/*
+*/
+//
 //		if (tr == 2 && pc == 0x5b6d) { printf("=%02x", a); tr = 1; }
 		
 
 	}
-*/
+
 	m_sys->m_fdc->tick();	// Check for FDC interrupt
 	m_sys->m_cps.tick();	// Check for CPS8256 interrupt
 	m_sys->m_daisy.tick();	// Check for daisywheel interrupt
@@ -219,7 +318,7 @@ void JoyceZ80::EDFE(void)
 			setHL(0); 
 			break;
 		case 0x0A:		/* A=10: Access DOS filesystem */
-			jdos(this);
+			if (jdos(this, m_sys) == -1) goodbye(99);
 			break;
 		case 0x0B:
 			fid_char(this);	/* A=11: Screen control FID */
@@ -334,6 +433,8 @@ unsigned JoyceZ80::out(unsigned long time,
 			   break;
 		case 0xFE: m_sys->m_locoLink.out(val);
 			   break;
+		case 0xFF: //write_pcwlink(this, val);
+			   //break;			
 		default:
 		joyce_dprintf("OUT 0x%02x, 0x%02x at PC=%04x\n", lo, val, pc);
 		break;
@@ -414,10 +515,11 @@ unsigned JoyceZ80::in(unsigned long time, unsigned long hi, unsigned long lo)
 			break;	
 		case 0xFE: return m_sys->m_locoLink.in();
 			   break;
-		
+		case 0xA7: case 0xA6:			/* Lightpen */
 		case 0xA0: case 0xA1: case 0xA2:	/* AMX mouse */
 		case 0xD0: case 0xD1: case 0xD2:	/* Kempston mouse */
 		case 0xD3: case 0xD4: return m_sys->m_activeMouse->in(lo);
+		case 0xFF: break;//return read_pcwlink(this);	
 	}
 	joyce_dprintf("IN(%02x) at PC=%04x\n", lo, pc);
 	return 0xFF;
