@@ -2,7 +2,7 @@
 
     JOYCE v2.1.6 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001-2,2005  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2,2005  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -24,6 +24,38 @@
 
 #ifdef HAVE_WINDOWS_H
 #include <SDL/SDL_thread.h>
+
+void ExpandError(DWORD err)
+{
+     LPTSTR s;
+     if(::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM,
+			NULL,
+			err,
+			0,
+			(LPTSTR)&s,
+			0,
+			NULL) == 0)
+	{ 
+		joyce_dprintf("Unknown error code %08x (%d)\n", err, 
+			LOWORD(err));
+	} 
+	else
+	{
+		LPTSTR p = strrchr(s, _T('\r'));
+		if(p != NULL)
+	   	{
+	     		*p = 0;
+		}
+		joyce_dprintf("Windows error %08x: %s\n", err, s);
+		::LocalFree(s);
+	}
+}
+
+
+
+//
+
 //
 // Win32 implementation of the abstract PcwComms class
 //
@@ -397,8 +429,10 @@ HANDLE PcwCommsW32::openComm(const char *port)
 		SetEvent(m_events[EV_START]);	
 		return h;
 	}
-	joyce_dprintf("Can't open %s, Win32 error %d\n", port,
-				GetLastError());
+	DWORD dwErr = GetLastError();
+	joyce_dprintf("Can't open %s: ", port);
+	ExpandError(dwErr);
+	joyce_dprintf("\n");
 	return openNUL(FILE_FLAG_OVERLAPPED);
 }
 
@@ -420,8 +454,10 @@ HANDLE PcwCommsW32::openNUL(int f)
 		return h;	
 	}
 
-	joyce_dprintf("Can't open NUL:, Win32 error %d\n", 
-				GetLastError());
+	DWORD dwErr = GetLastError();
+	joyce_dprintf("Can't open NUL: ");
+	ExpandError(dwErr);
+	joyce_dprintf("\n");
 	return h;
 }
 
@@ -469,7 +505,10 @@ void PcwCommsW32::openUp(void)
 				       FILE_ATTRIBUTE_NORMAL, NULL);
 		if (m_hFileIn == INVALID_HANDLE_VALUE) 
 		{
-			joyce_dprintf("Can't open %s: Win32 error %d\n", m_infile.c_str(), GetLastError());
+			DWORD dwErr = GetLastError();
+			joyce_dprintf("Can't open %s: ", m_infile.c_str());
+			ExpandError(dwErr);
+			joyce_dprintf("\n");
 			m_hFileIn = openNUL();
 		}
 		if (m_hFileOut == INVALID_HANDLE_VALUE)
@@ -479,7 +518,10 @@ void PcwCommsW32::openUp(void)
 					FILE_ATTRIBUTE_NORMAL, NULL);
 		if (m_hFileOut == INVALID_HANDLE_VALUE)
 		{
-			joyce_dprintf("Can't open %s: Win32 error %d\n", m_outfile.c_str(), GetLastError());
+			DWORD dwErr = GetLastError();
+			joyce_dprintf("Can't open %s: ", m_outfile.c_str());
+			ExpandError(dwErr);
+			joyce_dprintf("\n");
 			m_hFileOut = openNUL();
 		}
 	}
@@ -499,10 +541,25 @@ void PcwCommsW32::setParams(DCB *pDCB, int nbits)
 	if (m_stopBits == 4) pDCB->StopBits = TWOSTOPBITS;
 
 	pDCB->ByteSize = nbits;
-	
+	pDCB->fAbortOnError = 0;	
 }
 
 
+BOOL GetCommState_Retry(HANDLE hFile, LPDCB lpDCB)
+{
+	for (int n = 0; n < 2; n++)
+	{
+		BOOL result = GetCommState(hFile, lpDCB);
+		if (result) return true;
+
+		if (GetLastError() != ERROR_OPERATION_ABORTED)
+			return false;
+
+		ClearCommError(hFile, NULL, NULL);
+	}
+	return false;
+}
+
 
 
 void PcwCommsW32::setRX(int baud)
@@ -513,12 +570,29 @@ void PcwCommsW32::setRX(int baud)
 	memset(&dcb, 0, sizeof(dcb));
 	dcb.DCBlength = sizeof(dcb);
 
-	if (!m_rxComm) return;
-	if (m_hFileIn != INVALID_HANDLE_VALUE && GetCommState(m_hFileIn, &dcb))
+	if (!m_rxComm) 
+	{
+	//	COMM_LOG(( "Can't set RX baud rate: Not a comm port\n"));
+		return;
+	}
+	if (m_hFileIn == INVALID_HANDLE_VALUE)
+	{
+	//	COMM_LOG(("Couldn't set baud rate: invalid handle\n"));
+		return;
+	}
+	if (GetCommState_Retry(m_hFileIn, &dcb))
 	{	
 		dcb.BaudRate = baud2windows(baud);
+	//	COMM_LOG(( "Set RX baud rate to %d\n", dcb.BaudRate));
 	        SetCommState(m_hFileIn, &dcb);
 	}
+	else
+	{
+		DWORD dwErr = GetLastError(); 
+		joyce_dprintf("GetCommState() failed: ");
+		ExpandError(dwErr);
+		joyce_dprintf("\n");
+	}
 	PurgeComm(m_hFileIn,  PURGE_RXABORT | PURGE_RXCLEAR);
 }
 
@@ -530,12 +604,29 @@ void PcwCommsW32::setTX(int baud)
 	memset(&dcb, 0, sizeof(dcb));
 	dcb.DCBlength = sizeof(dcb);
 
-	if (!m_txComm) return;
-	if (m_hFileOut != INVALID_HANDLE_VALUE && GetCommState(m_hFileOut, &dcb))
+	if (!m_txComm) 
+	{
+//		COMM_LOG(( "Can't set TX baud rate: Not a comm port\n"));
+		return;
+	}
+	if (m_hFileOut == INVALID_HANDLE_VALUE)
+	{
+//		COMM_LOG(("Couldn't set baud rate: invalid handle\n"));
+		return;
+	}
+	if (GetCommState_Retry(m_hFileOut, &dcb))
 	{	
 		dcb.BaudRate = baud2windows(baud);
+//		COMM_LOG(( "Set TX baud rate to %d\n", dcb.BaudRate));
 	        SetCommState(m_hFileOut, &dcb);
 	}
+	else
+	{
+		DWORD dwErr = GetLastError();
+		joyce_dprintf("GetCommState() failed: ");
+		ExpandError(dwErr);
+		joyce_dprintf("\n");
+	}
 	PurgeComm(m_hFileOut,  PURGE_TXABORT | PURGE_TXCLEAR);
 }
 
@@ -561,12 +652,12 @@ void PcwCommsW32::setAllParams(void)
 
 	memset(&dcb, 0, sizeof(dcb));
 	dcb.DCBlength = sizeof(dcb);
-	if (m_rxComm && m_hFileIn != INVALID_HANDLE_VALUE && GetCommState(m_hFileIn, &dcb))
+	if (m_rxComm && m_hFileIn != INVALID_HANDLE_VALUE && GetCommState_Retry(m_hFileIn, &dcb))
 	{	
         	setParams(&dcb, m_rxbits);
 	        SetCommState(m_hFileIn, &dcb);
 	}
-	if (m_txComm && m_hFileOut != INVALID_HANDLE_VALUE && GetCommState(m_hFileOut, &dcb))
+	if (m_txComm && m_hFileOut != INVALID_HANDLE_VALUE && GetCommState_Retry(m_hFileOut, &dcb))
 	{	
         	setParams(&dcb, m_txbits);
 	        SetCommState(m_hFileOut, &dcb);
@@ -581,7 +672,7 @@ void PcwCommsW32::setTXbits(int b)
 	memset(&dcb, 0, sizeof(dcb));
 	dcb.DCBlength = sizeof(dcb);
 
-	if (m_txComm && m_hFileOut != INVALID_HANDLE_VALUE && GetCommState(m_hFileOut, &dcb))
+	if (m_txComm && m_hFileOut != INVALID_HANDLE_VALUE && GetCommState_Retry(m_hFileOut, &dcb))
 	{	
         	setParams(&dcb, m_txbits);
 	        SetCommState(m_hFileOut, &dcb);
@@ -596,7 +687,7 @@ void PcwCommsW32::setRXbits(int b)
 	dcb.DCBlength = sizeof(dcb);
 
 	m_rxbits = b;
-	if (m_rxComm && m_hFileIn != INVALID_HANDLE_VALUE && GetCommState(m_hFileIn, &dcb))
+	if (m_rxComm && m_hFileIn != INVALID_HANDLE_VALUE && GetCommState_Retry(m_hFileIn, &dcb))
 	{	
         	setParams(&dcb, m_rxbits);
 	        SetCommState(m_hFileIn, &dcb);
@@ -622,12 +713,18 @@ void PcwCommsW32::write(char c)
 
 	if (b) return;		// it worked
 
-	if (GetLastError() == ERROR_IO_PENDING) 
+	DWORD dwErr = GetLastError();
+	if (dwErr == ERROR_IO_PENDING) 
 	{
 		// waiting for results of write
 		m_wrpend = true;
 	}
-	else joyce_dprintf("PcwCommsW32::write() error 0x%x\n", GetLastError());
+	else 
+	{
+		joyce_dprintf("PcwCommsW32::write() ");
+		ExpandError(dwErr);
+		joyce_dprintf("\n");
+	}
 }
 
 
