@@ -2,7 +2,7 @@
 
     JOYCE v2.1.3 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001, 2003  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001, 2003  John Elliott <seasip.webmaster@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *************************************************************************/
 
@@ -72,7 +72,8 @@ PcwHDC::PcwHDC(PcwSystem *sys) : PcwDevice("hdc")
 {
 	int n;
 	char buf[20];
-		
+
+	m_all8mb = false;		
 	m_sys = sys;	
 	m_allowFloppy = false;
 	for (n = 0; n < 16; n++) 
@@ -118,13 +119,16 @@ UiEvent PcwHDC::getFolderType(UiDrawer *d, int &fmt, bool hd)
 		uim.add(new UiLabel  ("Setting up folder as PCW disc", d));
 		uim.add(new UiLabel  (" What format should be used? ", d));
 		uim.add(new UiSeparator(d));
-		uim.add(new UiSetting(SDLK_1, (fmt == 0), "  173k PCW format  ", d));
-		uim.add(new UiSetting(SDLK_2, (fmt == 1), "  169k CPC system  ", d));
-		uim.add(new UiSetting(SDLK_3, (fmt == 2), "  178k CPC data  ", d));
-		uim.add(new UiSetting(SDLK_4, (fmt == 3), "  192k PCW format  ", d));
-		uim.add(new UiSetting(SDLK_5, (fmt == 4), "  706k PCW format  ", d));
-		uim.add(new UiSetting(SDLK_6, (fmt == 5), "  784k PCW format  ", d));
-		uim.add(new UiSetting(SDLK_6, (fmt == 5), "  784k PCW format  ", d));
+		if (!m_all8mb)
+		{
+			uim.add(new UiSetting(SDLK_1, (fmt == 0), "  173k PCW format  ", d));
+			uim.add(new UiSetting(SDLK_2, (fmt == 1), "  169k CPC system  ", d));
+			uim.add(new UiSetting(SDLK_3, (fmt == 2), "  178k CPC data  ", d));
+			uim.add(new UiSetting(SDLK_4, (fmt == 3), "  192k PCW format  ", d));
+			uim.add(new UiSetting(SDLK_5, (fmt == 4), "  706k PCW format  ", d));
+			uim.add(new UiSetting(SDLK_6, (fmt == 5), "  784k PCW format  ", d));
+			uim.add(new UiSetting(SDLK_6, (fmt == 5), "  784k PCW format  ", d));
+		}
 		uim.add(new UiSetting(SDLK_7, (fmt == 6), "  8Mb MYZ80 format  ", d));
 		uim.add(new UiSeparator(d));
 		uim.add(new UiCommand(SDLK_o, "  OK  ", d));
@@ -297,7 +301,10 @@ UiEvent PcwHDC::setupDrive(UiDrawer *d, int drive)
 		uim.add(new UiSetting(SDLK_d, dsk, "  Disc file...  ",d));
 		uim.add(new UiSetting(SDLK_f, folder, "  Folder...  ",d));
 		uim.add(new UiSeparator(d));
-		uim.add(new UiSetting(SDLK_l, autoType, "  fLoppy drive  ", d));
+		if (!m_all8mb)
+		{
+			uim.add(new UiSetting(SDLK_l, autoType, "  fLoppy drive  ", d));
+		}
 		uim.add(new UiSetting(SDLK_h, myz80,    "  Hard drive  ", d));
 		uim.add(new UiSeparator(d));
 		uim.add(new UiCommand(SDLK_ESCAPE, "  EXIT  ", d));
@@ -378,6 +385,11 @@ UiEvent PcwHDC::settings(UiDrawer *d)
 		UiMenu uim(d);
 		uim.add(new UiLabel("  Emulated extra drives  ", d));	
 		uim.add(new UiSeparator(d));
+		uim.add(new UiSetting(SDLK_8, 
+			  m_all8mb, "  All drives use fixed 8Mb format  ", d));
+		uim.add(new UiLabel("  (this allows more than 6 drives  ", d));
+		uim.add(new UiLabel("   to be mapped under CP/M)", d));
+		uim.add(new UiSeparator(d));
 		for (n = 2; n < 16; n++)
 		{
 			SDLKey k = (SDLKey)(SDLK_a + n);
@@ -396,6 +408,9 @@ UiEvent PcwHDC::settings(UiDrawer *d)
                 if (uie == UIE_QUIT) return uie;
                 switch(uim.getKey(sel = uim.getSelected()))
                 {
+			case SDLK_8:
+				m_all8mb = !m_all8mb;
+				break;
 			case SDLK_ESCAPE:
 				reset();
 				return UIE_OK;
@@ -442,6 +457,9 @@ bool PcwHDC::parseDriveNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 
 bool PcwHDC::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
+	char *s = (char *)xmlGetProp(cur, (xmlChar *)"all8mb");
+	if (s) m_all8mb = atoi(s);
+
         // Settings for individual drives
         cur = cur->xmlChildrenNode;
         while (cur != NULL)
@@ -474,6 +492,8 @@ bool PcwHDC::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
         char buf[20];
         xmlNodePtr drv;
 
+	sprintf(buf, "%d", m_all8mb);
+	xmlSetProp(cur, (xmlChar *)"all8mb", (xmlChar *)buf);
         for (int n = 0; n < 16; n++)
         {
                 some = false;
@@ -827,7 +847,7 @@ void PcwHDC::fidFlush(Z80 *R)
 
 void PcwHDC::fidMessage(Z80 *R)	/* (v1.22) expand disc error message */
 {
-	char msg[94];
+	char msg[98];
 
 	sprintf(msg,"%c: %-92s", m_lastDrv + 'A', m_lastErr.c_str());
 
@@ -842,7 +862,7 @@ void PcwHDC::setupPars(Z80 *R)
 {	// or hard, and set HL IX IY accordingly 
 	int img = R->b;
 
-	if (isHard(m_drvName[img], m_drvType[img]))	// Fixed
+	if (m_all8mb || isHard(m_drvName[img], m_drvType[img]))	// Fixed
 	{
 		R->setHL(513);
 		R->ix = 0;
@@ -944,13 +964,72 @@ void PcwHDC::fidEms(Z80 *R)
 		++cdrv;
 		R->b = cdrv;
 		setupPars(R);
+/* The all-8mb hack: If we know all drives allocated by JOYCE will have 
+ * exactly the same geometry, we don't need a DPB for each one. PCW CP/M 
+ * allows up to 6 drives to be added, but the constraint is allocating the
+ * DPBs in common memory. Unfortunately there's no way to tell SVC_D_HOOK
+ * that it doesn't have to allocate the DPB; so let's do it by hand. */
+		if (m_all8mb)
+		{
+			all8mhack(R);
+
+		}
 		return;
 	}
 
 }
 
 
+void PcwHDC::all8mhack(Z80 *R)
+{
+	word drvtbl, dph, dpb;
+	
+/* 1. See if this is CP/M, by checking for the BIOS jumpblock */
+	for (int n = 0xFC00; n <= 0xFC60; n += 3)
+	{
+		if (m_sys->m_cpu->fetchB(n) != 0xC3) 
+		{
+			return;
+		}
+	}
+/* 2. Locate the DPH of the drive that was just added (E=letter) */
+	drvtbl = m_sys->m_cpu->fetchW(1 + m_sys->m_cpu->fetchW(0xFC43));
+	dph    = m_sys->m_cpu->fetchW(drvtbl+2*R->e);
+	dpb    = m_sys->m_cpu->fetchW(dph+12);
 
+/*	fprintf(stderr, "drvtbl=%04x dph=%04x dpb=%04x\n", drvtbl, dph, dpb);
+ *
+ * 3. Now locate the piece of code in the FID loader that allocates a new 
+ *   DPB... 
+ *    11 11 00    ld      de, 11h
+ *    cd 76 f2    call    svc_alloc_tpa
+ *    d2 37 f1    jp      nc, svc_d_hook_6
+ *    fd 75 10    ld      (iy+10h), l
+ *    fd 74 11    ld      (iy+11h), h
+ */
+	for (int n = 0xFFF0; n >= 0; n--)
+	{
+		if (m_sys->m_cpu->fetchB(n)    == 0x11 && 
+		    m_sys->m_cpu->fetchB(n+ 1) == 0x11 && 
+		    m_sys->m_cpu->fetchB(n+ 2) == 0x00 && 
+		    m_sys->m_cpu->fetchB(n+ 9) == 0xFD && 
+		    m_sys->m_cpu->fetchB(n+10) == 0x75 && 
+		    m_sys->m_cpu->fetchB(n+11) == 0x10 && 
+		    m_sys->m_cpu->fetchB(n+12) == 0xFD && 
+		    m_sys->m_cpu->fetchB(n+13) == 0x74 && 
+		    m_sys->m_cpu->fetchB(n+14) == 0x11)
+		{
+/*			fprintf(stderr, "Drive hook code at %04x\n", n); */
+
+			m_sys->m_cpu->storeB(n+3, 0x21); /* LD HL, DPB */
+			m_sys->m_cpu->storeW(n+4, dpb);
+			m_sys->m_cpu->storeB(n+6, 0x37); /* SCF */
+			m_sys->m_cpu->storeB(n+7, 0x00); /* NOP */
+			m_sys->m_cpu->storeB(n+8, 0x00); /* NOP */
+		}
+	}
+
+}	 
 
 
 static word to_al0(word blocks)
